#ifndef _ShadowMapping_H__
#define _ShadowMapping_H__

#include "common.h"
#include "glInterface.h"
#include "AxisAlignedBox3.h"
#include "Matrix4x4.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>



namespace CHCDemoEngine 
{ 

class FrameBufferObject;
class RenderTraverser;
class Vector3;
class Camera;
class Light;

/** This class implements a shadow mapping algorithm
*/
class ShadowMapping
{
public:
	/** constructor for a deferred shader taking the requested shadow size
	*/
	ShadowMapping(const AxisAlignedBox3 &sceneBox, int size);

	~ShadowMapping();
	/** Renders the shadowed scene
	*/
	void Render(RenderTraverser *traverser, Camera *cam);
	/** Initialises shadow mapping and loads the required shaders:
		This function has to be called only once.
	*/
	static void Init(CGcontext context);

//protected:

	void ComputeShadowMap(Light *light, RenderTraverser *traverser);

//protected:

	FrameBufferObject *mFbo;

	int mSize;

	AxisAlignedBox3 mSceneBox;

	Camera *mShadowCam;

	Matrix4x4 mLightViewMatrix;
	Matrix4x4 mLightProjectionMatrix;

	Light *mLight;
};

} // namespace 
#endif // _DeferredShader_H__